#ifndef __SHADERS_H__
#define __SHADERS_H__

#include <iostream>
#include "globals.h"

using namespace std;

void printInfoLog(char* headline, GLhandleARB obj)
{
  GLint infologLength = 0;
  GLint charsWritten  = 0;
  
  glGetObjectParameterivARB(obj, GL_OBJECT_INFO_LOG_LENGTH_ARB,
                            &infologLength);
  if (infologLength > 0)
    {
    char* infoLog = new char[infologLength];
    glGetInfoLogARB(obj, infologLength, &charsWritten, infoLog);
    cout << headline << " InfoLog:" << infoLog << endl;
    delete infoLog;
    }
}

GLhandleARB install_shaders() 
{
  GLhandleARB VS, FS, Prog;   // handles to objects
  GLint       vertCompiled, fragCompiled;    // status values
  GLint       linked;
  
  // Create a vertex shader object and a fragment shader object
  VS = glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
  FS = glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
  
  // Load source code strings into shaders
  
  glShaderSourceARB(VS, 1, &tri_vert_string, 0);
  glShaderSourceARB(FS, 1, &tri_frag_string, 0);
  
  // Compile the  vertex shader
  glCompileShaderARB(VS);
  glGetObjectParameterivARB(VS,
                            GL_OBJECT_COMPILE_STATUS_ARB, &vertCompiled);
  printInfoLog("Vertex program ", VS);  
  
  // Compile the fragment shader
  glCompileShaderARB(FS);
  glGetObjectParameterivARB(FS,
                            GL_OBJECT_COMPILE_STATUS_ARB, &fragCompiled);

  printInfoLog("Fragment program", FS);
  if (!vertCompiled || !fragCompiled)
    {
    cout << "Ooops. Shader compilation failed" << endl;
    return 0;
    }
  // Create a program object and attach the two compiled shaders
  Prog = glCreateProgramObjectARB();
  glAttachObjectARB(Prog, VS);
  glAttachObjectARB(Prog, FS);
  
  // Link the program object
  glLinkProgramARB(Prog);
  glGetObjectParameterivARB(Prog,
                            GL_OBJECT_LINK_STATUS_ARB, &linked);
  
  printInfoLog("Linking", Prog);
  if(!linked)
    {
    cout << "Ooops. Linking failed";
    return 0;
    }
  return Prog;
}

#endif
