const char * tri_vert_string =
"/*                                                                            \n"
"	Input: The vertex position and vertex attributes p1_3d and p2_3d which       \n"
"	are the positions of neighbouring vertices.                                  \n"
"                                                                              \n"
"	Output:   dist a vector of distances from the vertex to the three edges of   \n"
"	the triangle. Clearly only one of these distance is non-zero. For vertex 0   \n"
"	in a triangle dist = (distance to opposite edge, 0, 0) on exit. The distance \n"
"	is multiplied by w. This is to negate perspective correction.                \n"
"*/                                                                            \n"
"uniform vec2 WIN_SCALE;                                                       \n"
"attribute vec4 p1_3d;                                                         \n"
"attribute vec4 p2_3d;                                                         \n"
"                                                                              \n"
"varying vec3 dist;                                                            \n"
"void main(void)                                                               \n"
"{                                                                             \n"
"	 // We store the vertex id (0,1, or 2) in the w coord of the vertex          \n"
"	 // which then has to be restored to w=1.                                    \n"
"	 float swizz = gl_Vertex.w;                                                  \n"
"	 vec4 pos = gl_Vertex;                                                       \n"
"	 pos.w = 1.0;                                                                \n"
"                                                                              \n"
"	 // Compute the vertex position in the usual fashion.                        \n"
"   gl_Position = gl_ModelViewProjectionMatrix * pos;                          \n"
"	                                                                             \n"
"	 // p0 is the 2D position of the current vertex.                             \n"
"	 vec2 p0 = gl_Position.xy/gl_Position.w;                                     \n"
"                                                                              \n"
"	 // Project p1 and p2 and compute the vectors v1 = p1-p0                     \n"
"	 // and v2 = p2-p0                                                           \n"
"	 vec4 p1_3d_ = gl_ModelViewProjectionMatrix * p1_3d;                         \n"
"	 vec2 v1 = WIN_SCALE*(p1_3d_.xy / p1_3d_.w - p0);                            \n"
"                                                                              \n"
"	 vec4 p2_3d_ = gl_ModelViewProjectionMatrix * p2_3d;                         \n"
"	 vec2 v2 = WIN_SCALE*(p2_3d_.xy / p2_3d_.w - p0);                            \n"
"                                                                              \n"
"	 // Compute 2D area of triangle.                                             \n"
"	 float area2 = abs(v1.x*v2.y - v1.y * v2.x);                                 \n"
"                                                                              \n"
"   // Compute distance from vertex to line in 2D coords                       \n"
"   float h = area2/length(v1-v2);                                             \n"
"                                                                              \n"
"   // ---                                                                     \n"
"   // The swizz variable tells us which of the three vertices                 \n"
"   // we are dealing with. The ugly comparisons would not be needed if 	     \n"
"   // swizz was an int.                                                     	 \n"
"                                                                              \n"
"   if(swizz<0.1)                                                              \n"
"      dist = vec3(h,0,0);                                                     \n"
"   else if(swizz<1.1)                                                         \n"
"      dist = vec3(0,h,0);                                                     \n"
"   else                                                                       \n"
"      dist = vec3(0,0,h);                                                     \n"
"                                                                              \n"
"   // ----                                                                    \n"
"   // Quick fix to defy perspective correction                                \n"
"                                                                              \n"
"   dist *= gl_Position.w;                                                     \n"
"}                                                                             \n";
